package de.towerdefence.server.server;

import com.fasterxml.jackson.databind.ObjectMapper;
import de.towerdefence.server.oas.ServerApi;
import de.towerdefence.server.oas.models.PlayerRegistrationData;
import de.towerdefence.server.oas.models.ServerHealth;
import de.towerdefence.server.player.Player;
import de.towerdefence.server.player.PlayerRepository;
import de.towerdefence.server.player.PlayerService;
import jakarta.servlet.http.HttpServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;

import java.security.NoSuchAlgorithmException;
import java.util.Optional;

@Controller
@RequestMapping("${openapi.api.base-path:/api/v1}")
public class ServerApiController implements ServerApi {

    @Autowired
    private PlayerRepository playerRepository;
    @Autowired
    private PlayerService playerService;

    @Override
    public Optional<ObjectMapper> getObjectMapper() {
        return Optional.empty();
    }

    @Override
    public Optional<HttpServletRequest> getRequest() {
        return Optional.empty();
    }

    @Override
    public ResponseEntity<Void> playerRegister(PlayerRegistrationData body) {
        if(playerRepository.existsByUsername(body.getUsername())){
            return new ResponseEntity<>(HttpStatus.CONFLICT);
        }
        Player newPlayer = new Player();
        newPlayer.setUsername(body.getUsername());
        try{
            playerService.setPassword(newPlayer, body.getPassword());
        } catch (NoSuchAlgorithmException exception){
            return new ResponseEntity<>(HttpStatus.INTERNAL_SERVER_ERROR);
        }
        playerRepository.save(newPlayer);
        return new ResponseEntity<>(HttpStatus.CREATED);
    }

    @Override
    public ResponseEntity<ServerHealth> serverGetHealthcheck() {
        ServerHealth health = new ServerHealth();
        health.setOkay(true);
        return ResponseEntity.ok(health);
    }
}
